import os
import platform

"""
@ModuleName: PathConfig
@Description: 基础路径以及系统相关信息保存文件
@Author: Beier
@Time: 2022/2/23 22:47
"""
#
# # 这个方法可以获取相关系统环境，可以使用这个方法配置保存路径，但是需要注意linux下的环境key和window下的key是不一样的，需要分系统
# for i in os.environ.keys():
#     print(i + "：" + os.environ[i])
# 操作系统
PLATFORM = platform.system()
# HOME路径
if PLATFORM == "Windows":
    HOME_PATH = os.environ["USERPROFILE"]
else:
    HOME_PATH = os.environ["HOME"]
# print(HOME_PATH)
# 根路径
ROOT_PATH = os.path.dirname(os.path.dirname(__file__))
# 配置项路径
CONFIG_PATH = ROOT_PATH + os.sep + "config"
# 工具类路径
UTIL_PATH = ROOT_PATH + os.sep + "util"
# 实体类路径
ENTITY_PATH = ROOT_PATH + os.sep + "entity"
# 异常类路径
EXCEPTION_PATH = ROOT_PATH + os.sep + "exception"
# 日志路径
LOGGING_PATH = ROOT_PATH + os.sep + "log"
# js逆向类路径
JS_PATH = ROOT_PATH + os.sep + "js"
# 请求器路径
REQUESTER_PATH = ROOT_PATH + os.sep + "requester"
# 解析器路径
PARSER_PATH = ROOT_PATH + os.sep + "parser"
# 时刻表路径（用于服务器部署）
SCHEDULE_PATH = ROOT_PATH + os.sep + "schedule"
# print(SCHEDULE_PATH)
# 默认结果保存路径
RESULT_PATH = ROOT_PATH + os.sep + "result"



"""
@ModuleName: ErrorLogger
@Description: ERROR日志类，负责非自定义异常记录
@Author: Beier
@Time: 2022/2/23 23:16
"""
from log.BaseLogger import BaseLogging


class ErrorLogger(BaseLogging):
    def __init__(self):
        BaseLogging.__init__(self, name="error", level=BaseLogging.ERROR)



"""
@ModuleName: InfoLogger
@Description: INFO日志类，负责各类INFO输出【如：各种result，中间提示等】
@Author: Beier
@Time: 2022/2/23 23:16
"""
from log.BaseLogger import BaseLogging


class InfoLogger(BaseLogging):
    def __init__(self, name):
        BaseLogging.__init__(self, name=name, level=BaseLogging.INFO)



"""
@ModuleName: BaseUtil
@Description: 工具包基础类，进行基础配置供子类调用
@Author: Beier
@Time: 2022/2/23 22:47
"""

import os
import config.PathConfig as pc
from ruamel import yaml
import json


class BaseUtil:
    def __init__(self):
        f_r_config = open(pc.CONFIG_PATH + os.sep + "config.yaml", mode="r", encoding="utf-8")
        self.yaml = yaml.YAML()
        self.config = self.yaml.load(f_r_config)
        f_r_agent = open(pc.CONFIG_PATH + os.sep + "user_agent.json", mode="r", encoding="utf-8")
        self.user_agent = json.load(f_r_agent)

    def _update_config(self, key: str, value):
        if key in self.config:
            f_w = open(pc.CONFIG_PATH + os.sep + "config.yaml", mode="w", encoding="utf-8")
            self.config[key] = value
            self.yaml.dump(data=self.config, stream=f_w)
        else:
            from exception.ConfigException import ConfigException
            raise ConfigException(f"config file update fail with do not find the key：{key}")



"""
@ModuleName: SbiqerException
@Description: 基础异常类
@Author: Beier
@Time: 2022/2/23 23:00
"""
from log.BaseLogger import BaseLogging


class SbiqerException(Exception):
    def __init__(self, message):
        logger = BaseLogging("exception", BaseLogging.ERROR).get_logger()
        logger.error(message)
        # print(message)



"""
@ModuleName: SaveUtil
@Description: 保存工具类，暴露给用户进行调用
@Author: Beier
@Time: 2022/2/23 22:45
"""
from util.BaseUtil import BaseUtil
from util.ConfigLoader import ConfigLoader
# import config.PathConfig as pc

cl = ConfigLoader()


class SaveUtil(BaseUtil):
    def __init__(self):
        BaseUtil.__init__(self)

    def get_save_path(self):
        return cl.get_result_path()

    def set_save_path(self, path):
        self._update_config("result_path", path)



"""
@ModuleName: MultUtil
@Description: 杂项类，用来存放如进度条等无处安放的小插件，暂定内部组件皆为静态方法，以方便调用
@Author: Beier
@Time: 2022/2/23 22:44
"""


class MultUtil(object):
    # def __init__(self):
    #     pass

    @staticmethod
    def process_bar(percent, start_str='', end_str='', total_length=0):
        bar = '\r' + start_str + ''.ljust(int(percent * total_length), "=") + '> {:0>5.2f}%|'.format(
            percent * 100) + end_str
        print(bar, end='', flush=True)



"""
@ModuleName: ConfigException
@Description: 配置异常类
@Author: Beier
@Time: 2022/2/23 23:02
"""
from exception.SbiqerException import SbiqerException


class ConfigException(SbiqerException):
    def __init__(self, message):
        SbiqerException.__init__(self, message)



"""
@ModuleName: BaseLogger
@Description: 基础日志类，负责各类INFO输出【如：自定义异常日志等】，以及ERROR，WARNING，INFO日志类继承
@Author: Beier
@Time: 2022/2/23 23:15
"""
import logging
import os
from util.ConfigLoader import ConfigLoader

cl = ConfigLoader()


class BaseLogging:
    CRITICAL = logging.CRITICAL
    ERROR = logging.ERROR
    WARNING = logging.WARNING
    INFO = logging.INFO
    DEBUG = logging.DEBUG

    def __init__(self, name, level):
        logging.basicConfig(format="%(asctime)s - %(levelname)s: %(message)s",
                            datefmt="%Y-%m-%d %H:%M:%S",
                            level=level,
                            filemode='a')
        self.__logger = logging.getLogger(name)
        logging_formatter = logging.Formatter(fmt="%(asctime)s - %(levelname)s: %(message)s",
                                              datefmt='%Y-%m-%d %H:%M:%S')
        file_handler = logging.FileHandler(filename=cl.get_log_path() + os.sep + name + ".log", mode="a",
                                           encoding="utf-8")
        file_handler.setFormatter(logging_formatter)
        self.__logger.addHandler(file_handler)

    def get_logger(self):
        return self.__logger


if __name__ == '__main__':
    logger = BaseLogging("test", BaseLogging.INFO).get_logger()
    logger.error("测试")



# from exception.ConfigException import ConfigException
# from exception.SbiqerException import SbiqerException
import traceback
# from log.BaseLogger import BaseLogging
# from log.ErrorLogger import ErrorLogger
from util.SaveUtil import SaveUtil
import sys
from pprint import pprint
import requests
pprint(sys.builtin_module_names)
# save_util = SaveUtil()
# print(sys.argv)
# save_util.set_save_path("D:\pycharm\PyCharm 2020.1.1\workplace\sbiqer")
# a = 0
# b = 1
# try:
#     c = b / a
# except:
#     logger=ErrorLogger().get_logger()
#     logger.exception(traceback.print_exc())
# if a == 0:
#     raise SbiqerException("配置文件出错，请重新更改配置文件")

# import yaml
# import config.PathConfig as pc
# import os
# import json
#
# config = open(pc.CONFIG_PATH + os.sep + "config.yaml", mode="r", encoding="utf-8")
# config_json = open(pc.CONFIG_PATH + os.sep + "config.json", mode="r", encoding="utf-8")
# cfg = config.read()
# json_line = json.load(config_json)
# yaml_line = yaml.load(stream=cfg, Loader=yaml.FullLoader)
# print(type(yaml_line))
# print(type(json_line))



"""
@ModuleName: WarningLogger
@Description: WARNING日志类，负责各类WARNING输出【如：各类使用到的如requests库的内置WARNING等】
@Author: Beier
@Time: 2022/2/23 23:16
"""
from log.BaseLogger import BaseLogging


class WarningLogger(BaseLogging):
    def __init__(self):
        BaseLogging.__init__(self, name="warning", level=BaseLogging.WARNING)



"""
@ModuleName: ConfigLoader
@Description: 配置加载器，用于加载config.json配置项
@Author: Beier
@Time: 2022/2/23 22:46
"""

from util.BaseUtil import BaseUtil
import config.PathConfig as pc
import random
import os
import json


class ConfigLoader(BaseUtil):
    IOS = 1
    WEB = 2
    ANDROID = 3

    def __init__(self):
        BaseUtil.__init__(self)

    def get_log_path(self):
        '''
        :return: 如果config.yaml文件中log_path属性非空字符串，即返回相应路径，若为空字符串，返回默认路径
        '''
        return pc.LOGGING_PATH if self.config["log_path"] == "" else self.config["log_path"]

    def get_result_path(self):
        return pc.RESULT_PATH if self.config["result_path"] == "" else self.config["result_path"]

    def get_random_agent(self, platform: int):
        """

        :param platform: 平台，具体种类见类属性
        :return: 根据所给的平台返回相应的user-agent
        """
        if platform == self.IOS:
            return random.choice(self.user_agent["user_Agent_ios"])
        elif platform == self.WEB:
            return random.choice(self.user_agent["user_Agent_web"])
        elif platform == self.ANDROID:
            return random.choice(self.user_agent["user_Agent_android"])
        else:
            from exception.ConfigException import ConfigException
            raise ConfigException("error user-Agent key.")

    def get_random_cookie(self, cookie_pool_path: str):
        from exception.ConfigException import ConfigException
        '''
        根据给出的cookie池文件路径，读取cookie池，并随机返回
        :param cookie_pool_path: 该路径底部文件需是json文件
        :return: 随机从cookie池中抽取一个
        '''
        if os.path.splitext(cookie_pool_path)[-1] != ".json":
            raise ConfigException("can not find a json file as cookie pool")
        else:
            f_r = open(cookie_pool_path, "r", encoding="utf-8")
            cookie_json = json.load(f_r)
            if "cookie_pool" not in cookie_json or type(cookie_json["cookie_pool"]) != list:
                raise ConfigException("check your cookie_pool file format according to document: https://www.baidu.com/")
            else:
                return random.choice(cookie_json["cookie_pool"])


if __name__ == '__main__':
    cl = ConfigLoader()
    print(cl.get_random_cookie(r"D:\pycharm\PyCharm 2020.1.1\workplace\sbiqer\config\cookie_pool.json"))



#!/usr/bin/python
# encoding: utf-8
from setuptools import setup, find_packages

setup(
    name="sbiqer",
    version="0.1",
    license="MIT Licence",

    url="https://github.com/srx-2000/sbiqer",
    author="beier",
    author_email="1601684622@qq.com",

    packages=find_packages(),
    include_package_data=True,
    platforms="any",
    install_requires=[]
)


#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
@ModuleName: ThreadPool
@Description: 线程池
@Author: 佚名
@Time: 2022/2/23 22:50
"""

import queue
import threadpool
import contextlib
import time
import traceback

StopEvent = object()


class ThreadPool(object):

    def __init__(self, max_num):
        self.q = queue.Queue()  # 存放任务的队列
        self.max_num = max_num  # 最大线程并发数

        self.terminal = False  # 如果为True 终止所有线程，不再获取新任务
        self.generate_list = []  # 已经创建的线程
        self.free_list = []  # 闲置的线程
        # self.run_sum_time=0

    def run(self, func, args, callback=None):
        """
        线程池执行一个任务
        :param func: 任务函数
        :param args: 任务函数所需参数
        :param callback: 任务执行失败或成功后执行的回调函数，回调函数有两个参数1、任务函数执行状态；2、任务函数返回值（默认为None，即：不执行回调函数）
        :return: 如果线程池已经终止，则返回True否则None
        """

        if len(self.free_list) == 0 and len(self.generate_list) < self.max_num:  # 无空闲线程和不超过最大线程数
            self.generate_thread()  # 创建线程
        w = (func, args, callback,)  # 保存参数为元组
        self.q.put(w)  # 添加到任务队列
        # self.run_sum_time+=1

    def generate_thread(self):
        """
        创建一个线程
        """
        t = threadpool.Thread(target=self.call)
        t.start()

    def call(self):
        """
        循环去获取任务函数并执行任务函数
        """
        current_thread = threadpool.currentThread  # 获取当前线程对象
        self.generate_list.append(current_thread)  # 添加到已创建线程里

        event = self.q.get()  # 获取任务
        while event != StopEvent:  # 如果不为停止信号

            func, arguments, callback = event  # 分别取值，
            try:
                result = func(*arguments)  # 运行函数，把结果赋值给result
                status = True  # 运行结果是否正常
            except Exception as e:
                status = False  # 不正常
                result = traceback.format_exc()  # 结果为错误信息

            if callback is not None:  # 是否有回调函数
                try:
                    callback(status, result)  # 执行回调函数
                except Exception as e:
                    # print("回调函数出错："+str(traceback.format_exc()))
                    pass
            if self.terminal:  # 默认为False ，如果调用terminal方法
                event = StopEvent  # 停止信号
            else:
                # self.free_list.append(current_thread) #执行完毕任务，添加到闲置列表
                # event = self.q.get()    #获取任务
                # self.free_list.remove(current_thread) #获取到任务之后，从闲置里删除
                with self.worker_state(self.free_list, current_thread):
                    event = self.q.get()


        else:
            self.generate_list.remove(current_thread)  # 如果收到终止信号，就从已创建的列表删除

    def close(self):  # 终止线程
        num = len(self.generate_list)  # 获取总已创建的线程
        while num:
            self.q.put(StopEvent)  # 添加停止信号，有几个线程就添加几个
            num -= 1

    # 终止线程（清空队列）
    def terminate(self):

        self.terminal = True  # 更改为True，

        while self.generate_list:  # 如果有已创建线程存活
            self.q.put(StopEvent)  # 有几个就发几个信号
        self.q.empty()  # 清空队列

    @contextlib.contextmanager
    def worker_state(self, free_list, current_thread):
        free_list.append(current_thread)
        try:
            yield
        finally:
            free_list.remove(current_thread)
